@@ -22,6 +22,7 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <string.h>
+#include <X11/keysym.h>
 #include "main.h"
 #include "utils.h"
 #include "history.h"
@@ -107,13 +108,18 @@ static void edit_selected()
 {
   GtkTreeIter sel_iter;
   GtkTreeSelection* search_selection = gtk_tree_view_get_selection((GtkTreeView*)treeview_search);
-  /* This helps prevent multiple instances and checks if there's anything selected */
-  if (gtk_tree_selection_get_selected(search_selection, NULL, &sel_iter))
-  {
+  /* This checks if there's anything selected */
+  gint selected_count = gtk_tree_selection_count_selected_rows(search_selection);
+  if (selected_count > 0) {
     /* Create clipboard buffer and set its text */
     gint selected_item_nr;
-    gtk_tree_model_get((GtkTreeModel*)search_list, &sel_iter, 0, &selected_item_nr, -1);
+    GList *selected_rows = gtk_tree_selection_get_selected_rows(search_selection, NULL);
+    GList *row_loop = g_list_first(selected_rows);
+    selected_item_nr = atoi((gchar*)gtk_tree_path_to_string(row_loop->data));
+    g_list_foreach(selected_rows, (GFunc)gtk_tree_path_free, NULL);
+    g_list_free(selected_rows);
     GSList* element = g_slist_nth(history, selected_item_nr);
+    GSList* elementafter = element->next;
     GString* s_selected_item = g_string_new((gchar*)element->data);
     GtkTextBuffer* clipboard_buffer = gtk_text_buffer_new(NULL);
     gtk_text_buffer_set_text(clipboard_buffer, s_selected_item->str, -1);
@@ -151,13 +157,13 @@ static void edit_selected()
       /* Delete any duplicate */
       delete_duplicate(gtk_text_buffer_get_text(clipboard_buffer, &start, &end, TRUE));
 
-      /* Insert new element before the old one */
-      history = g_slist_insert_before(history, element->next,
-                    g_strdup(gtk_text_buffer_get_text(clipboard_buffer, &start, &end, TRUE)));
-
       /* Remove old entry */
       history = g_slist_remove(history, element->data);
 
+      /* Insert new element where the old one was */
+      history = g_slist_insert_before(history, elementafter,
+                    g_strdup(gtk_text_buffer_get_text(clipboard_buffer, &start, &end, TRUE)));
+
       if(selected_item_nr == 0)
       {
         GtkClipboard* clipboard;
@@ -173,21 +179,32 @@ static void edit_selected()
   }
 }
 
+static void add_iter(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *piter, gpointer userdata)
+{
+  GArray *sel = (GArray*)userdata;
+  GtkTreeIter iter = *piter;
+  g_array_append_val(sel, iter);
+}
+
 /* Called when Remove is selected from Manage dialog */
 static void remove_selected()
 {
-  GtkTreeIter sel_iter;
   GtkTreeSelection* search_selection = gtk_tree_view_get_selection((GtkTreeView*)treeview_search);
-  /* This checks if there's anything selected */
-  if (gtk_tree_selection_get_selected(search_selection, NULL, &sel_iter))
-  {
-    /* Get item to delete */
-    gint selected_item_nr;
-    gtk_tree_model_get((GtkTreeModel*)search_list, &sel_iter, 0, &selected_item_nr, -1);
-    GSList* element = g_slist_nth(history, selected_item_nr);
-    /* Remove entry */
-    history = g_slist_remove(history, element->data);
-    search_history();
+  gint selected_count = gtk_tree_selection_count_selected_rows(search_selection);
+  if (selected_count > 0) {
+    gint i;
+    GtkListStore *store = GTK_LIST_STORE(gtk_tree_view_get_model((GtkTreeView*)treeview_search));
+    GArray *sel = g_array_new(FALSE, FALSE, sizeof(GtkTreeIter));
+    gtk_tree_selection_selected_foreach(search_selection, add_iter, sel);
+    gtk_tree_selection_unselect_all(search_selection);
+    for (i = 0; i < sel->len; i++) {
+        gint remove_item;
+        GtkTreeIter *iter = &g_array_index(sel, GtkTreeIter, i);
+        gtk_tree_model_get((GtkTreeModel*)search_list, iter, 0, &remove_item, -1);
+        history = g_slist_remove(history, g_slist_nth_data(history, remove_item));
+        gtk_list_store_remove(store, iter);
+    }
+    g_array_free(sel, TRUE);
   }
 }
 
@@ -232,13 +249,17 @@ static void remove_all_selected(gpointer user_data)
 
 static void search_doubleclick()
 {
-  GtkTreeIter sel_iter;
   GtkTreeSelection* search_selection = gtk_tree_view_get_selection((GtkTreeView*)treeview_search);
   /* Check if selected */
-  if (gtk_tree_selection_get_selected(search_selection, NULL, &sel_iter))
-  {
+  gint selected_count = gtk_tree_selection_count_selected_rows(search_selection);
+  if (selected_count == 1) {
     gint selected_item_nr;
-    gtk_tree_model_get((GtkTreeModel*)search_list, &sel_iter, 0, &selected_item_nr, -1);
+    GArray *sel = g_array_new(FALSE, FALSE, sizeof(GtkTreeIter));
+    gtk_tree_selection_selected_foreach(search_selection, add_iter, sel);
+    gtk_tree_selection_unselect_all(search_selection);
+    GtkTreeIter *iter = &g_array_index(sel, GtkTreeIter, 0);
+    gtk_tree_model_get((GtkTreeModel*)search_list, iter, 0, &selected_item_nr, -1);
+    g_array_free(sel, TRUE);
     GSList *element = g_slist_nth(history, selected_item_nr);
     GtkClipboard* prim = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
     GtkClipboard* clip = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
@@ -247,7 +268,7 @@ static void search_doubleclick()
   }
 }
 
-static gint search_click(GtkWidget *widget, GdkEventButton *event, GtkWidget *search_window)
+static gboolean search_click(GtkWidget *widget, GdkEventButton *event, GtkWidget *search_window)
 {
   if(event->type==GDK_2BUTTON_PRESS || event->type==GDK_3BUTTON_PRESS)
   {
@@ -257,14 +278,38 @@ static gint search_click(GtkWidget *widget, GdkEventButton *event, GtkWidget *se
   return FALSE;
 }
 
-static gint search_key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
+static gboolean search_key_released(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
 {
-  /* Check if [Return] key was pressed */
-  if ((event->keyval == 0xff0d) || (event->keyval == 0xff8d))
-    search_history();
+  search_history();
   return FALSE;
 }
 
+static gboolean treeview_key_pressed(GtkWidget *widget, GdkEventKey *event, GtkWidget *search_window)
+{
+  switch (event->keyval) {
+    case XK_Escape:
+    case XK_Home:
+    case XK_Up:
+    case XK_Down:
+    case XK_Page_Up:
+    case XK_Page_Down:
+    case XK_End:
+    case XK_Shift_L:
+    case XK_Shift_R:
+    case XK_Control_L:
+    case XK_Control_R:
+      return FALSE;
+    case XK_Return:
+      search_doubleclick();
+      gtk_widget_destroy(search_window);
+      break;
+    case XK_Delete:
+      remove_selected();
+      break;
+  }
+  return TRUE;
+}
+
 void search_window_response(GtkDialog *dialog, gint response_id, gpointer user_data)
 {
   if(response_id < 0)
@@ -301,7 +346,7 @@ gboolean show_search()
   gtk_box_pack_start((GtkBox*)vbox_search, hbox, FALSE, FALSE, 0);
   search_entry = gtk_entry_new();
   gtk_box_pack_end((GtkBox*)hbox, search_entry, TRUE, TRUE, 0);
-  g_signal_connect((GObject*)search_entry, "key-press-event", (GCallback)search_key_pressed, NULL);
+  g_signal_connect((GObject*)search_entry, "key-release-event", (GCallback)search_key_released, NULL);
 
   /* Build the exclude treeview */
   GtkWidget* scrolled_window_search = gtk_scrolled_window_new(
@@ -336,8 +381,9 @@ gboolean show_search()
   g_signal_connect((GObject*)close_button, "clicked", (GCallback)search_history, NULL);
 
   GtkTreeSelection* search_selection = gtk_tree_view_get_selection((GtkTreeView*)treeview_search);
-  gtk_tree_selection_set_mode(search_selection, GTK_SELECTION_BROWSE);
+  gtk_tree_selection_set_mode(search_selection, GTK_SELECTION_MULTIPLE);
   g_signal_connect((GObject*)treeview_search, "button_press_event", (GCallback)search_click, search_dialog);
+  g_signal_connect((GObject*)treeview_search, "key-press-event", (GCallback)treeview_key_pressed, search_dialog);
 
   g_signal_connect((GtkDialog*)search_dialog, "response", (GCallback)search_window_response, search_dialog);
   gtk_widget_show_all(vbox_search);
